#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <curses.h>

#define MAX_COMPONENTS 512
#define MAX_WINDATAS   128

struct component {
	char * id;
	char * name;
	char * action;
	char * value;
	int size, y, x;
};

struct component components[MAX_COMPONENTS];
int last_component = -1;

struct windata {
	int y1, x1;
	int y2, x2;
};

struct windata windatas[MAX_WINDATAS];
int last_windata = -1;

void action_text(struct component *);
void action_select(struct component *);
void action_next(struct component *);

void draw_component(struct component *);
void help();

struct action_handler {
	char * name;
	void (*handler)(struct component *);
};

struct action_handler action_handlers[] = {
	{ "text",	action_text    },
	{ "select",	action_select  },
	{ "next",	action_next    },
	{ NULL, NULL} };

#define DEBUG_READARGV 1
#define DEBUG_COMP_POS 2
int debug = 255;

#define COL_BG 1
#define COL_WF 2
#define COL_WS 3
#define COL_FG 4
#define COL_SL 5
#define COL_CM 6

void draw_char(int ch, int no_change_color) {
	int winch = 0;
	int c, x, y;

	getyx(stdscr, y, x);

	for (c=0; c <= last_windata; c++) {
		if (windatas[c].x1 == x) {
			if (windatas[c].y1 == y) winch = ACS_ULCORNER;
			else if (windatas[c].y2 == y) winch = ACS_LLCORNER;
			else if (windatas[c].y1 < y &&
			         windatas[c].y2 > y) winch = ACS_VLINE;
		}
		else if (windatas[c].x2 == x) {
			if (windatas[c].y1 == y) winch = ACS_URCORNER;
			else if (windatas[c].y2 == y) winch = ACS_LRCORNER;
			else if (windatas[c].y1 < y &&
			         windatas[c].y2 > y) winch = ACS_VLINE;
		}
		else if ( (windatas[c].y1 == y || windatas[c].y2 == y) &&
		           windatas[c].x1 < x && windatas[c].x2 > x)
							winch = ACS_HLINE;
		else if ( windatas[c].y1 < y && windatas[c].y2 > y &&
		          windatas[c].x1 < x && windatas[c].x2 > x)
							winch = ' ';
	}

	if (! no_change_color) {
		if (winch) {
			color_set(COL_WF, NULL); 
			ch = ch == ' ' ? winch : ch;
		} else color_set(COL_BG, NULL);
	}

	addch(ch);
}

int main(int argc, char ** argv) {
	int c, input;
	int window_bg = 0;
	int offset_y=0, offset_x=0;

	struct component * current;
	char * init_component = NULL;
	char * template = argv[3];
	char * current_id = NULL;

	/* Init nCurses
	*/
	initscr();
	start_color();
	raw();
	noecho();
	keypad(stdscr, TRUE);

	init_pair(COL_BG, COLOR_CYAN,  COLOR_BLUE);
	init_pair(COL_WF, COLOR_BLACK, COLOR_WHITE);
	init_pair(COL_WS, COLOR_CYAN,  COLOR_BLACK);
	init_pair(COL_FG, COLOR_BLACK, COLOR_CYAN);
	init_pair(COL_SL, COLOR_BLUE,  COLOR_YELLOW);
	init_pair(COL_CM, COLOR_WHITE, COLOR_GREEN);

	/* Parse options
	*/
	for (c=1; argv[c][0] == '-' && c < argc; c++) {
		if (!strcmp(argv[c], "--init")) {
			init_component = argv[++c];
		} else if (!strcmp(argv[c], "--winyx")) {
			last_windata++;
			windatas[last_windata].y1=atoi(argv[++c]) + offset_y;
			windatas[last_windata].x1=atoi(argv[++c]) + offset_x;
			windatas[last_windata].y2=atoi(argv[++c]) + offset_y;
			windatas[last_windata].x2=atoi(argv[++c]) + offset_x;
		} else if (!strcmp(argv[c], "--size")) {
			int max_y, max_x;
			getmaxyx(stdscr, max_y, max_x);

			offset_y = (max_y-atoi(argv[++c])) / 2;
			offset_x = (max_x-atoi(argv[++c])) / 2;

			if (offset_y < 0) offset_y = 0;
			if (offset_x < 0) offset_x = 0;
		} else if (!strcmp(argv[c], "--windowbg")) {
			window_bg = 1;
		} else
			help();
	}
	if (c >= argc) help();

	/* Parse component list
	*/
	template = argv[c++];
	for (; c<argc; c++) {
		char * options;

		last_component++;
		components[last_component].id = strtok(argv[c], " \t");
		if (components[last_component].id == NULL)
						{ last_component--; continue; }

		components[last_component].name = strtok(NULL, " \t");
		if (components[last_component].name == NULL)
						{ last_component--; continue; }

		options = strtok(NULL, " \t");
		if (options == NULL) { last_component--; continue; }
		components[last_component].size = atoi(options);

		components[last_component].action = strtok(NULL, " \t");
		if (components[last_component].action == NULL)
						{ last_component--; continue; }

		components[last_component].value  =
			components[last_component].action + 1 +
			strlen(components[last_component].action);
		while (components[last_component].value[0] == ' ' ||
		       components[last_component].value[0] == '\t')
					components[last_component].value++;

		if (debug & DEBUG_READARGV) {
			fprintf(stderr, "# Component #%d in Argv #%d:\n",
						last_component, c);
			fprintf(stderr, "#  ID     = %s\n",
				components[last_component].id);
			fprintf(stderr, "#  Name   = %s\n",
				components[last_component].name);
			fprintf(stderr, "#  Action = %s\n",
				components[last_component].action);
			fprintf(stderr, "#  Value  = %s\n",
				components[last_component].value);
			fprintf(stderr, "#  Size   = %d\n",
				components[last_component].size);
			fprintf(stderr, "\n");
		}
	}

	/* Draw Background
	*/
	if (window_bg) {
		color_set(COL_WF, NULL);
		bkgd(COLOR_PAIR(COL_WF));
	} else {
		color_set(COL_BG, NULL);
		bkgd(COLOR_PAIR(COL_BG));
	}

	/* Draw Windows
	*/
	for (c=0; c <= last_windata; c++) {
		struct windata * thisw = windatas + c;
		int c1, c2;

		for (c1 = thisw->y1; c1 <= thisw->y2; c1++) {
			move(c1, thisw->x1);
			color_set(COL_WF, NULL);

			if (c1 == thisw->y1) addch(ACS_ULCORNER);
			else if (c1 == thisw->y2) addch(ACS_LLCORNER);
			else addch(ACS_VLINE);

			if ( c1 == thisw->y1 || c1 == thisw->y2)
				for (c2 = thisw->x1+1;
					c2 < thisw->x2; c2++) addch(ACS_HLINE);
			else
				for (c2 = thisw->x1+1;
					c2 < thisw->x2; c2++) addch(' ');

			if (c1 == thisw->y1) addch(ACS_URCORNER);
			else {
				if (c1 == thisw->y2) addch(ACS_LRCORNER);
				else addch(ACS_VLINE);

				color_set(COL_WS, NULL);
				addch(' '); addch(' ');
			}
		}
		move(thisw->y2+1, thisw->x1+2);
		for (c1 = thisw->x1+2; c1 <= thisw->x2+2; c1++) addch(' ');
	}

	/* Draw Template
	*/
	move(offset_y, offset_x);
	if (window_bg) color_set(COL_WF, NULL);
	else color_set(COL_BG, NULL);
	for (c=0; template[c]; c++) {
		if ( template[c] == '%' ) {
		    if ( template[c+1] == '%' ) {
			c++; addch(template[c]);
		    } else {
			char name[100];
			struct component * thisc = NULL;
			int c2=0, size=2;

			for (c++; template[c] && template[c] != '%'; c++) {
				if (template[c] != ' ' && template[c] != '\t')
						name[c2++] = template[c];
				size++;
			}
			name[c2]=0;

			for (c2=0; c2 <= last_component; c2++)
				if (!strcmp(name, components[c2].id))
					thisc = components + c2;
			if (thisc == NULL) continue;

			if (thisc->size == 0) thisc->size = size;
			getyx(stdscr, thisc->y, thisc->x);
			move(thisc->y, thisc->x + thisc->size);
			if (debug & DEBUG_COMP_POS) {
				fprintf(stderr, "# Comp %s:\n", thisc->id);
				fprintf(stderr, "#   Y    = %d\n",
								thisc->y);
				fprintf(stderr, "#   X    = %d\n",
								thisc->x);
				fprintf(stderr, "#   SIZE = %d\n",
								thisc->size);
				fprintf(stderr, "\n");
			}
		    }
		} else if ( template[c] == '\n' ) {
			int y, x;
			getyx(stdscr, y, x);
			move(y+1, offset_x);
		} else
			draw_char(template[c], window_bg);
	}

	/* Draw Components
	*/
	color_set(COL_FG, NULL);
	for (c=0; c <= last_component; c++)
		draw_component(components + c);

	/* Select Component
	*/
	current = components;
	if (init_component != NULL)
		for (c=0; c <= last_component; c++)
			if (!strcmp(components[c].id, init_component))
				current = components + c;
	while ( 1 ) {
		color_set(COL_SL, NULL);
		draw_component(current);

		if (can_change_color()) {
			move(0, 0); refresh();
			leaveok(stdscr, TRUE);
			if ( (input = getch()) == 0 ) break;
			leaveok(stdscr, FALSE);
		} else {
			move(current->y, current->x);
			if ( (input = getch()) == 0 ) break;
		}

		color_set(COL_FG, NULL);
		draw_component(current);

		for (c=0; c <= last_component; c++)
			if (components[c].id[0] == input &&
			    components[c].id[1] == 0) {
				current_id = current->id;
				current = components + c;
				input = '\n';
			}
		if (input == '\n' || input == '\r') {
			color_set(COL_CM, NULL);
			if (!can_change_color())
				attron(A_BOLD);
			move(current->y, current->x);
			for (c=0; c < current->size &&
					current->value[c]; c++)
						addch(current->value[c]);
			for (; c < current->size; c++) addch(' ');
			move(current->y, current->x);

			for (c=0; action_handlers[c].name; c++) {
				int len = strlen(action_handlers[c].name);
				if ( !strncmp(action_handlers[c].name,
				              current->action, len) &&
				     ( current->action[len] == 0 ||
				       current->action[len] == ',' )) {
						action_handlers[c].handler(
							current );
						break;
				}
			}

			if (current_id == NULL)
						current_id = current->id;
			fprintf(stderr, "ffdialog_id='%s'\n",
						current_id);
			fprintf(stderr, "ffdialog_name='%s'\n",
						current->name);
			fprintf(stderr, "ffdialog_value='%s'\n",
						current->value);
			break;
		}

		if (input == '\t' &&
			current < components+last_component) current++;

		if (input == KEY_BTAB && current > components) current--;

		if (input == KEY_UP || input == KEY_DOWN) {
			int current_y=current->y;
			int current_x=current->x;
			struct component * best = current;
			int diff, best_diff=10000;

			for (current = components;
			     current <= components+last_component;
			     current++) {
				diff = current_y - current->y;

				if (input == KEY_DOWN && diff < 0 &&
				    diff*-1 < best_diff ) {
						best_diff = diff*-1;
						best = current;
				}
				if (input == KEY_UP   && diff > 0 &&
				    diff < best_diff ) {
						best_diff = diff;
						best = current;
				}
			}
			current_y = best->y;
			best_diff = 10000;

			for (current = components;
			     current <= components+last_component;
			     current++) {
				diff = current_x - current->x;
				if (diff < 0) diff*=-1;

				if (current->y == current_y &&
				    diff < best_diff) {
					best_diff = diff;
					best = current;
				}
			}
			current = best;
		}

		if (input == KEY_LEFT || input == KEY_RIGHT) {
			int current_y=current->y;
			int current_x=current->x;
			struct component * best = current;
			int diff, best_diff=10000;

			for (current = components;
			     current <= components+last_component;
			     current++) {
				diff = current_x - current->x;

				if (input == KEY_RIGHT && diff < 0 &&
				    current_y == current->y &&
				    diff*-1 < best_diff ) {
						best_diff = diff*-1;
						best = current;
				}
				if (input == KEY_LEFT && diff > 0 &&
				    current_y == current->y &&
				    diff < best_diff ) {
						best_diff = diff;
						best = current;
				}
			}
			current = best;
		}
		move(current->y, current->x);
	}

	/* Finish
	*/
	noraw();
	endwin();

	return 0;
}

void action_text(struct component * current) {
	static char newvalue[100];
	int position = 0;
	int insert = 1;
	int input, c;

	strcpy(newvalue, current->value);

	while ( (input = getch()) != 0 ) {
		if (input >= 32 && input < 127 && insert == 0) {
			if (! newvalue[position])
				newvalue[position+1] = 0;
			newvalue[position] = input;
			position++;
		}
		if (input >= 32 && input < 127 && insert == 1) {
			char temp[100];

			for (c=0; c<position && newvalue[c]; c++)
						temp[c] = newvalue[c];
			temp[c++] = input;
			for (; newvalue[c-1]; c++)
						temp[c] = newvalue[c-1];
			temp[current->size]=0;
			temp[c]=0;

			strcpy(newvalue, temp);
			position++;
		}
		if ((input == KEY_BACKSPACE || input == 8) && position > 0) {
			position--; input = KEY_DC;
		}
		if (input == KEY_DC) {
			char temp[100];
			for (c=0; c<position && newvalue[c]; c++)
						temp[c] = newvalue[c];
			for (; newvalue[c] && newvalue[c+1]; c++)
						temp[c] = newvalue[c+1];
			temp[c]=0;
			strcpy(newvalue, temp);
			if (!newvalue[position]) position--;
		}

		if (input == KEY_RIGHT) position++;
		if (input == KEY_LEFT)  position--;

		if (input == KEY_IC) insert = !insert;
		if (input == 033) return;
		if (input == '\n' || input == '\r') break;

		if (position < 0) position=0;
		if (position >= current->size) position=current->size-1;

		if (! newvalue[position]) {
			newvalue[position] = ' ';
			newvalue[position+1] = 0;
		}

		move(current->y, current->x);
		for (c=0; c < current->size && newvalue[c]; c++)
							addch(newvalue[c]);
		for (; c < current->size; c++) addch(' ');
		move(current->y, current->x + position);
	}
	current->value = newvalue;
}

void action_select(struct component * current) {
	action_next(current);  // FIXME
}

void action_next(struct component * current) {
	char * first = NULL;
	char * value = NULL;
	char * this;

	strtok(current->action, ",");
	while ( (this = strtok(NULL, ",")) != NULL ) {
		if (first == NULL) first = this;
		if (!strcmp(current->value, this)) {
			value = strtok(NULL, ",");
			break;
		}
	}
	
	current->value = value;
	if ( current->value == NULL )
		current->value = first;
}

void help() {
	printf(
"\n"
" Usage: $0\n"
"	[ --init <ID of first component> ]\n"
"	<Template>\n"
"	<Component 1> [ <Component 2> [ .. ] ]\n"
"\n"
	);
	exit(1);
}

void draw_component(struct component * current) {
	int c;
	move(current->y, current->x);
	for (c=0; c < current->size && current->value[c]; c++)
					addch(current->value[c]);
	for (; c < current->size; c++) addch(' ');
	move(current->y, current->x);
}

