#!/bin/sh

if [ "${1#-}" != "$1" -o -z "$1" ] ; then
	echo
	echo "BacteriuM GUI v0.2 - An editor for bacterium *.synth files"
	echo
	echo "Usage: $0 myconfig.synth"
	echo
	echo "WARNING: Make backups of your *.synth files before you open"
	echo "them in this editor!"
	echo
	exit 1
else
	file="$1" ; touch "$1"
fi

ffdialog="./bacterium_ffd"

dspfile=/dev/null
for x in /dev/dsp /dev/sound/dsp ; do
	[ -e $x ] && dspfile=$x
done

# Set default values
#
for osc in 1 2 3 4 5 6 ; do
	eval "osc_${osc}_wave=Sin"
	eval "osc_${osc}_freq=C3"
	eval "osc_${osc}_tran='*'"
	eval "osc_${osc}_oenv="0,0,0,0,0,0"
	eval "osc_${osc}_fenv="0,0,0,0,0,0"
	for osc2 in 1 2 3 4 5 6 ; do
		eval "osc_${osc}_mm_${osc2}='-'"
	done
	eval "osc_${osc}_tomi='*'"
done
#
lpass_fenv="0,0,10000,0,20000,20000"
lpass_renv="0,0,10000,0,0,0"
hpass_fenv="0,0,10000,0,0,0"
hpass_renv="0,0,10000,0,0,0"
#
sh=0,0,0
dl=0,0,0,0
ff=aeiou,0
ch=0
ph=0
vl=1
ov=0
tp=0

# Read File
#
while read name para ; do
  case "$name" in
    master_volume)
	vl=$para ;;
    overdrive_level)
	ov=$para ;;

    osc_waveforms)
	for osc in 1 2 3 4 5 6 ; do
		val=${para%%,*}
		para=${para#*,}
		case "$val" in
			0) val=Sin ;;
			1) val=Tri ;;
			2) val=Saw ;;
			3) val=Squ ;;
			4) val=Noi ;;
		esac
		eval "osc_${osc}_wave=$val"
	done
	;;

    osc_frequencies)
	for osc in 1 2 3 4 5 6 ; do
		val=${para%%,*}
		para=${para#*,}
		eval "osc_${osc}_freq=$val"
	done
	;;

    transpose)
	for osc in tp 1 2 3 4 5 6 ; do
		val=${para%%,*}
		para=${para#*,}
		if [ $osc = tp ] ; then tp=$val
		else
			[ "$val" = 0 ] && val='-'
			[ "$val" = 1 ] && val='*'
			eval "osc_${osc}_tran='$val'"
		fi
	done
	;;

    osc_envelope)
	osc=$(( ${para%%,*} + 1 ))
	para=${para#*,}
	eval "osc_${osc}_oenv='$para'"
	;;

    frq_envelope)
	osc=$(( ${para%%,*} + 1 ))
	para=${para#*,}
	eval "osc_${osc}_fenv='$para'"
	;;

    modulation_matrix)
	for osc in 1 2 3 4 5 6 ; do
		read para
		for osc2 in 1 2 3 4 5 6 ; do
			val=${para%%,*}
			para=${para#*,}
			case "$val" in
				0) val="-" ;;
				1) val="r" ;;
				2) val="R" ;;
				3) val="a" ;;
				4) val="f" ;;
				5) val="h" ;;
				6) val="p" ;;
				7) val="s" ;;
			esac
			eval "osc_${osc2}_mm_${osc}=$val"
		done
		case "$para" in
			0) eval "osc_${osc}_tomi='-'" ;;
			1) eval "osc_${osc}_tomi='*'" ;;
		esac
	done
	;;

    filter_envelope)
	filter=${para%%,*}
	para=${para#*,}
	case "$filter" in
		0) eval "lpass_fenv='$para'" ;;
		1) eval "lpass_renv='$para'" ;;
		2) eval "hpass_fenv='$para'" ;;
		3) eval "hpass_renv='$para'" ;;
	esac
	;;

    formantfilter)
	ff=$para ;;

    's&h')
	sh=$para ;;

    delay)
	dl=$para ;;

    chorus)
	ch=$para ;;

    phaser)
	ph=$para ;;
  esac
done < $file

## MAIN LOOP BEGIN
##
while true ; do

## DISPLAY THE GUI
##
if $ffdialog --init "$ffdialog_id" --size 24 80 --windowbg \
'           Wav      Freq       Trans          Freq. Envelope
 OSC 1 -> [%o1w%] -> [%o1f%] -> [%o1t%] -> [%o1fe%] --> A1
 OSC 2 -> [%o2w%] -> [%o2f%] -> [%o2t%] -> [%o2fe%] --> A2
 OSC 3 -> [%o3w%] -> [%o3f%] -> [%o3t%] -> [%o3fe%] --> A3
 OSC 4 -> [%o4w%] -> [%o4f%] -> [%o4t%] -> [%o4fe%] --> A4
 OSC 5 -> [%o5w%] -> [%o5f%] -> [%o5t%] -> [%o5fe%] --> A5
 OSC 6 -> [%o6w%] -> [%o6f%] -> [%o6t%] -> [%o6fe%] --> A6

              Osc. Envelope              O-1 O-2 O-3 O-4 O-5 O-6    Mix
 A1 -> [%o1oe%] -> [%o11%] [%o12%] [%o13%] [%o14%] [%o15%] [%o16%] -> [%o1m%] -> Mix
 A2 -> [%o2oe%] -> [%o21%] [%o22%] [%o23%] [%o24%] [%o25%] [%o26%] -> [%o2m%] -> Mix
 A3 -> [%o3oe%] -> [%o31%] [%o32%] [%o33%] [%o34%] [%o35%] [%o36%] -> [%o3m%] -> Mix
 A4 -> [%o4oe%] -> [%o41%] [%o42%] [%o43%] [%o44%] [%o45%] [%o46%] -> [%o4m%] -> Mix
 A5 -> [%o5oe%] -> [%o51%] [%o52%] [%o53%] [%o54%] [%o55%] [%o56%] -> [%o5m%] -> Mix
 A6 -> [%o6oe%] -> [%o61%] [%o62%] [%o63%] [%o64%] [%o65%] [%o66%] -> [%o6m%] -> Mix

                      Freq. Envelope                 Resonance Envelope
 Lowpass ----> [%lpfe                     %] --> [%lpre                     %]
 Highpass ---> [%hpfe                     %] --> [%hpre                     %]

 S&H: [%sh      %]   Chorus: [%ch %]   Volume: [%vl  %]   Delay: [%dl       %]
 Transpose: [%tp%]   Phaser: [%ph %]   Overdr: [%ov  %]   Formant: [%ff     %]

 [%s%]  [%p%]  [%v%]                                       [%h%]   [%q%]' \
"o1w	osc_1_wave	3	next,Sin,Tri,Saw,Squ,Noi $osc_1_wave" \
"o1f	osc_1_freq	7	text			$osc_1_freq" \
"o1t	osc_1_tran	1	next,-,*		$osc_1_tran" \
"o1fe	osc_1_fenv	30	text			$osc_1_fenv" \
"o1oe	osc_1_oenv	28	text			$osc_1_oenv" \
"o11	osc_1_mm_1	1	next,-,r,R,a,f,h,p,s	$osc_1_mm_1" \
"o12	osc_1_mm_2	1	next,-,r,R,a,f,h,p,s	$osc_1_mm_2" \
"o13	osc_1_mm_3	1	next,-,r,R,a,f,h,p,s	$osc_1_mm_3" \
"o14	osc_1_mm_4	1	next,-,r,R,a,f,h,p,s	$osc_1_mm_4" \
"o15	osc_1_mm_5	1	next,-,r,R,a,f,h,p,s	$osc_1_mm_5" \
"o16	osc_1_mm_6	1	next,-,r,R,a,f,h,p,s	$osc_1_mm_6" \
"o1m	osc_1_tomi	1	next,-,*		$osc_1_tomi" \
\
"o2w	osc_2_wave	3	next,Sin,Tri,Saw,Squ,Noi $osc_2_wave" \
"o2f	osc_2_freq	7	text			$osc_2_freq" \
"o2t	osc_2_tran	1	next,-,*		$osc_2_tran" \
"o2fe	osc_2_fenv	30	text			$osc_2_fenv" \
"o2oe	osc_2_oenv	28	text			$osc_2_oenv" \
"o21	osc_2_mm_1	1	next,-,r,R,a,f,h,p,s	$osc_2_mm_1" \
"o22	osc_2_mm_2	1	next,-,r,R,a,f,h,p,s	$osc_2_mm_2" \
"o23	osc_2_mm_3	1	next,-,r,R,a,f,h,p,s	$osc_2_mm_3" \
"o24	osc_2_mm_4	1	next,-,r,R,a,f,h,p,s	$osc_2_mm_4" \
"o25	osc_2_mm_5	1	next,-,r,R,a,f,h,p,s	$osc_2_mm_5" \
"o26	osc_2_mm_6	1	next,-,r,R,a,f,h,p,s	$osc_2_mm_6" \
"o2m	osc_2_tomi	1	next,-,*		$osc_2_tomi" \
\
"o3w	osc_3_wave	3	next,Sin,Tri,Saw,Squ,Noi $osc_3_wave" \
"o3f	osc_3_freq	7	text			$osc_3_freq" \
"o3t	osc_3_tran	1	next,-,*		$osc_3_tran" \
"o3fe	osc_3_fenv	30	text			$osc_3_fenv" \
"o3oe	osc_3_oenv	28	text			$osc_3_oenv" \
"o31	osc_3_mm_1	1	next,-,r,R,a,f,h,p,s	$osc_3_mm_1" \
"o32	osc_3_mm_2	1	next,-,r,R,a,f,h,p,s	$osc_3_mm_2" \
"o33	osc_3_mm_3	1	next,-,r,R,a,f,h,p,s	$osc_3_mm_3" \
"o34	osc_3_mm_4	1	next,-,r,R,a,f,h,p,s	$osc_3_mm_4" \
"o35	osc_3_mm_5	1	next,-,r,R,a,f,h,p,s	$osc_3_mm_5" \
"o36	osc_3_mm_6	1	next,-,r,R,a,f,h,p,s	$osc_3_mm_6" \
"o3m	osc_3_tomi	1	next,-,*		$osc_3_tomi" \
\
"o4w	osc_4_wave	3	next,Sin,Tri,Saw,Squ,Noi $osc_4_wave" \
"o4f	osc_4_freq	7	text			$osc_4_freq" \
"o4t	osc_4_tran	1	next,-,*		$osc_4_tran" \
"o4fe	osc_4_fenv	30	text			$osc_4_fenv" \
"o4oe	osc_4_oenv	28	text			$osc_4_oenv" \
"o41	osc_4_mm_1	1	next,-,r,R,a,f,h,p,s	$osc_4_mm_1" \
"o42	osc_4_mm_2	1	next,-,r,R,a,f,h,p,s	$osc_4_mm_2" \
"o43	osc_4_mm_3	1	next,-,r,R,a,f,h,p,s	$osc_4_mm_3" \
"o44	osc_4_mm_4	1	next,-,r,R,a,f,h,p,s	$osc_4_mm_4" \
"o45	osc_4_mm_5	1	next,-,r,R,a,f,h,p,s	$osc_4_mm_5" \
"o46	osc_4_mm_6	1	next,-,r,R,a,f,h,p,s	$osc_4_mm_6" \
"o4m	osc_4_tomi	1	next,-,*		$osc_4_tomi" \
\
"o5w	osc_5_wave	3	next,Sin,Tri,Saw,Squ,Noi $osc_5_wave" \
"o5f	osc_5_freq	7	text			$osc_5_freq" \
"o5t	osc_5_tran	1	next,-,*		$osc_5_tran" \
"o5fe	osc_5_fenv	30	text			$osc_5_fenv" \
"o5oe	osc_5_oenv	28	text			$osc_5_oenv" \
"o51	osc_5_mm_1	1	next,-,r,R,a,f,h,p,s	$osc_5_mm_1" \
"o52	osc_5_mm_2	1	next,-,r,R,a,f,h,p,s	$osc_5_mm_2" \
"o53	osc_5_mm_3	1	next,-,r,R,a,f,h,p,s	$osc_5_mm_3" \
"o54	osc_5_mm_4	1	next,-,r,R,a,f,h,p,s	$osc_5_mm_4" \
"o55	osc_5_mm_5	1	next,-,r,R,a,f,h,p,s	$osc_5_mm_5" \
"o56	osc_5_mm_6	1	next,-,r,R,a,f,h,p,s	$osc_5_mm_6" \
"o5m	osc_5_tomi	1	next,-,*		$osc_5_tomi" \
\
"o6w	osc_6_wave	3	next,Sin,Tri,Saw,Squ,Noi $osc_6_wave" \
"o6f	osc_6_freq	7	text			$osc_6_freq" \
"o6t	osc_6_tran	1	next,-,*		$osc_6_tran" \
"o6fe	osc_6_fenv	30	text			$osc_6_fenv" \
"o6oe	osc_6_oenv	28	text			$osc_6_oenv" \
"o61	osc_6_mm_1	1	next,-,r,R,a,f,h,p,s	$osc_6_mm_1" \
"o62	osc_6_mm_2	1	next,-,r,R,a,f,h,p,s	$osc_6_mm_2" \
"o63	osc_6_mm_3	1	next,-,r,R,a,f,h,p,s	$osc_6_mm_3" \
"o64	osc_6_mm_4	1	next,-,r,R,a,f,h,p,s	$osc_6_mm_4" \
"o65	osc_6_mm_5	1	next,-,r,R,a,f,h,p,s	$osc_6_mm_5" \
"o66	osc_6_mm_6	1	next,-,r,R,a,f,h,p,s	$osc_6_mm_6" \
"o6m	osc_6_tomi	1	next,-,*		$osc_6_tomi" \
\
"lpfe	lpass_fenv	0	text		$lpass_fenv" \
"lpre	lpass_renv	0	text		$lpass_renv" \
"hpfe	hpass_fenv	0	text		$hpass_fenv" \
"hpre	hpass_renv	0	text		$hpass_renv" \
\
"sh	sh		0	text		$sh" \
"ch	ch		0	text		$ch" \
"vl	vl		0	text		$vl" \
"dl	dl		0	text		$dl" \
"tp	tp		0	text		$tp" \
"ph	ph		0	text		$ph" \
"ov	ov		0	text		$ov" \
"ff	ff		0	text		$ff" \
\
"s	#s		5	button		Samle" \
"p	#p		4	button		Play" \
"v	#v		4	button		View" \
"h	#h		4	button		Help" \
"q	#q		4	button		Quit" 2> bacterium.tmp
then
	. bacterium.tmp ; rm -f bacterium.tmp
	eval "$ffdialog_name=\"\$ffdialog_value\""
else
	echo "ERROR in running bacterium_ffd (the ncurses helper app)."
	cat bacterium.tmp ; rm -f bacterium.tmp
	exit 1
fi

# Output
#
{
	cat << EOT
##
## This file is auto-generated by Clifford Wolf's BacteriuM GUI.
## Please report bug to <clifford@clifford.at>.
##
## BacteriuM is written by Stefan Fendt.
##
EOT
	cat <<- EOT
		master_volume $vl
		overdrive_level $ov
	EOT

	echo -n "osc_waveforms "
	for osc in 1 2 3 4 5 6 ; do
		[ $osc != 1 ] && echo -n ","
		eval "x=\$osc_${osc}_wave"
		case $x in
			Tri) echo -n 1 ;;
			Saw) echo -n 2 ;;
			Squ) echo -n 3 ;;
			Noi) echo -n 4 ;;
			*)   echo -n 0 ;;
		esac
	done
	echo

	echo -n "osc_frequencies "
	for osc in 1 2 3 4 5 6 ; do
		[ $osc != 1 ] && echo -n ","
		eval "echo -n \$osc_${osc}_freq"
	done
	echo

	echo -n "transpose $tp,"
	for osc in 1 2 3 4 5 6 ; do
		[ $osc != 1 ] && echo -n ","
		case "$y" in
			-) echo -n 0 ;;
			*) echo -n 1 ;;
		esac
	done
	echo

	for osc in 1 2 3 4 5 6 ; do
		eval "echo osc_envelope $(($osc-1)),\$osc_${osc}_oenv"
		eval "echo frq_envelope $(($osc-1)),\$osc_${osc}_fenv"
	done

	echo modulation_matrix
	for osc in 1 2 3 4 5 6 ; do
		for x in 1 2 3 4 5 6 ; do
			eval "y=\$osc_${x}_mm_${osc}"
			case "$y" in
				r) echo -n 1, ;;
				R) echo -n 2, ;;
				a) echo -n 3, ;;
				f) echo -n 4, ;;
				h) echo -n 5, ;;
				p) echo -n 6, ;;
				s) echo -n 7, ;;
				*) echo -n 0, ;;
			esac
		done
		eval "y=\$osc_${osc}_tomi"
		case "$y" in
			-) echo 0 ;;
			*) echo 1 ;;
		esac
	done

	echo "filter_envelope 0,$lpass_fenv"
	echo "filter_envelope 1,$lpass_renv"
	echo "filter_envelope 2,$hpass_fenv"
	echo "filter_envelope 3,$hpass_renv"

	echo "formantfilter $ff"
	echo "s&h $sh"
	echo "delay $dl"
	echo "chorus $ch"
	echo "phaser $ph"
} > $file

# Actions
#
case "$ffdialog_name" in
    '#s')
	BacteriuM $file ${file%.synth}.wav
	echo -n "Press ENTER to continue ..." ; read
	;;

    '#p')
	trap "fuser -k $dspfile" INT
	BacteriuM $file $dspfile
	echo -n "Press ENTER to continue ..." ; read
	trap - INT
	;;

    '#v')
	${PAGER:-less} $file
	;;

    '#h')
	${PAGER:-less} /usr/doc/bacterium/examples/TEMPLATE.SYNTH
	;;

    '#q')
	exit 0
	;;
esac

done
##
## MAIN LOOP END

